package cn.reghao.oss.console.app.db.mapper;

import cn.reghao.jutil.jdk.db.BaseMapper;
import cn.reghao.oss.console.app.model.po.UploadChannel;

import java.util.List;

/**
 * @author reghao
 * @date 2024-07-02 18:15:27
 */
public interface UploadChannelRepository extends BaseMapper<UploadChannel> {
    int countByCreateBy(int createBy);
    List<UploadChannel> findByNodeIdAndCreateBy(int nodeId, int createBy);
    UploadChannel findByChannelIdAndCreateBy(int channelId, int createBy);
    UploadChannel findByPrefixAndCreateByAndNodeId(String prefix, int createBy, int nodeId);
    UploadChannel findByPrefixAndNodeId(String prefix, int nodeId);
    UploadChannel findById(int id);
    List<UploadChannel> findByCreateBy(int createBy);
}
